@@ -1,5 +1,3 @@
-/* $Cambridge: exim/exim-src/exim_monitor/em_globals.c,v 1.17 2009/11/16 19:50:36 nm4 Exp $ */
-
 /*************************************************
 *                Exim Monitor                    *
 *************************************************/
@@ -209,6 +207,8 @@ uschar *smtp_active_hostname   = NULL;
 BOOL    split_spool_directory  = FALSE;
 uschar *spool_directory        = US SPOOL_DIRECTORY;
 int     string_datestamp_offset=-1;
+int     string_datestamp_length= 0;
+int     string_datestamp_type  = -1;
 
 BOOL    timestamps_utc         = FALSE;
 BOOL    tls_certificate_verified = FALSE;
