@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "exim.h"
@@ -15,9 +15,9 @@
 /* All the directories that Exim ever creates for itself are within the spool
 directory as defined by spool_directory. We are prepared to create as many as
 necessary from that directory downwards, inclusive. However, directory creation
-can also be required in appendfile. The making function therefore has a parent
-argument, below which the new directories are to go. It can be NULL if the name
-is absolute.
+can also be required in appendfile and sieve filters. The making function
+therefore has a parent argument, below which the new directories are to go. It
+can be NULL if the name is absolute.
 
 If a non-root uid has been specified for exim, and we are currently running as
 root, ensure the directory is owned by the non-root id if the parent is the
@@ -34,10 +34,12 @@ Returns:    panic on failure if panic is set; otherwise return FALSE;
 */
 
 BOOL
-directory_make(uschar *parent, uschar *name, int mode, BOOL panic)
+directory_make(const uschar *parent, const uschar *name,
+               int mode, BOOL panic)
 {
 BOOL use_chown = parent == spool_directory && geteuid() == root_uid;
-uschar *p, *slash;
+uschar *p;
+const uschar *slash;
 int c = 1;
 struct stat statbuf;
 uschar buffer[256];
@@ -45,7 +47,7 @@ uschar buffer[256];
 if (parent == NULL)
   {
   p = buffer + 1;
-  slash = parent = US"";
+  slash = parent = CUS"";
   }
 else
   {
@@ -72,13 +74,13 @@ while (c != 0 && *p != 0)
 
     /* Set the ownership if necessary. */
 
-    if (use_chown) Uchown(buffer, exim_uid, exim_gid);
+    if (use_chown) (void)Uchown(buffer, exim_uid, exim_gid);
 
     /* It appears that any mode bits greater than 0777 are ignored by
     mkdir(), at least on some operating systems. Therefore, if the mode
     contains any such bits, do an explicit mode setting. */
 
-    if ((mode & 0777000) != 0) Uchmod(buffer, mode);
+    if ((mode & 0777000) != 0) (void)Uchmod(buffer, mode);
     }
   *p++ = c;
   }
