@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Exim gets and frees all its store through these functions. In the original
@@ -31,10 +31,15 @@ The following different types of store are recognized:
   after accepting a message when multiple messages are received by a single
   process. Resetting happens at some other times as well, usually fairly
   locally after some specific processing that needs working store.
+
+. There is a separate pool (POOL_SEARCH) that is used only for lookup storage.
+  This means it can be freed when search_tidyup() is called to close down all
+  the lookup caching.
 */
 
 
 #include "exim.h"
+#include "memcheck.h"
 
 
 /* We need to know how to align blocks of data for general use. I'm not sure
@@ -53,6 +58,11 @@ suitably aligned. */
 
 #define STORE_BLOCK_SIZE 8192
 
+/* store_reset() will not free the following block if the last used block has
+less than this much left in it. */
+
+#define STOREPOOL_MIN_SIZE 256
+
 /* Structure describing the beginning of each big block. */
 
 typedef struct storeblock {
@@ -74,10 +84,10 @@ even if the length is zero (which is used for getting a point to reset to). */
 
 int store_pool = POOL_PERM;
 
-static storeblock *chainbase[2] = { NULL, NULL };
-static storeblock *current_block[2] = { NULL, NULL };
-static void *next_yield[2] = { NULL, NULL };
-static int yield_length[2] = { -1, -1 };
+static storeblock *chainbase[3] = { NULL, NULL, NULL };
+static storeblock *current_block[3] = { NULL, NULL, NULL };
+static void *next_yield[3] = { NULL, NULL, NULL };
+static int yield_length[3] = { -1, -1, -1 };
 
 /* pool_malloc holds the amount of memory used by the store pools; this goes up
 and down as store is reset or released. nonpool_malloc is the total got by
@@ -91,7 +101,7 @@ static int nonpool_malloc = 0;
 NULL. This enables string_cat() to optimize its store handling for very long
 strings. That's why the variable is global. */
 
-void *store_last_get = NULL;
+void *store_last_get[3] = { NULL, NULL, NULL };
 
 
 
@@ -131,28 +141,46 @@ if (size > yield_length[store_pool])
   {
   int length = (size <= STORE_BLOCK_SIZE)? STORE_BLOCK_SIZE : size;
   int mlength = length + ALIGNED_SIZEOF_STOREBLOCK;
-  storeblock *newblock;
+  storeblock *newblock = NULL;
 
-  pool_malloc += mlength;           /* Used in pools */
-  nonpool_malloc -= mlength;        /* Exclude from overall total */
+  /* Sometimes store_reset() may leave a block for us; check if we can use it */
 
-  newblock = store_malloc(mlength);
-  newblock->next = NULL;
-  newblock->length = length;
+  if (current_block[store_pool] != NULL &&
+      current_block[store_pool]->next != NULL)
+    {
+    newblock = current_block[store_pool]->next;
+    if (newblock->length < length)
+      {
+      /* Give up on this block, because it's too small */
+      store_free(newblock);
+      newblock = NULL;
+      }
+    }
 
-  if (chainbase[store_pool] == NULL) chainbase[store_pool] = newblock;
-    else current_block[store_pool]->next = newblock;
+  /* If there was no free block, get a new one */
+
+  if (newblock == NULL)
+    {
+    pool_malloc += mlength;           /* Used in pools */
+    nonpool_malloc -= mlength;        /* Exclude from overall total */
+    newblock = store_malloc(mlength);
+    newblock->next = NULL;
+    newblock->length = length;
+    if (chainbase[store_pool] == NULL) chainbase[store_pool] = newblock;
+      else current_block[store_pool]->next = newblock;
+    }
 
   current_block[store_pool] = newblock;
-  yield_length[store_pool] = length;
+  yield_length[store_pool] = newblock->length;
   next_yield[store_pool] =
     (void *)((char *)current_block[store_pool] + ALIGNED_SIZEOF_STOREBLOCK);
+  VALGRIND_MAKE_MEM_NOACCESS(next_yield[store_pool], yield_length[store_pool]);
   }
 
 /* There's (now) enough room in the current block; the yield is the next
 pointer. */
 
-store_last_get = next_yield[store_pool];
+store_last_get[store_pool] = next_yield[store_pool];
 
 /* Cut out the debugging stuff for utilities, but stop picky compilers from
 giving warnings. */
@@ -167,16 +195,17 @@ DEBUG(D_memory)
     debug_printf("---%d Get %5d\n", store_pool, size);
   else
     debug_printf("---%d Get %6p %5d %-14s %4d\n", store_pool,
-      store_last_get, size, filename, linenumber);
+      store_last_get[store_pool], size, filename, linenumber);
   }
 #endif  /* COMPILE_UTILITY */
 
+VALGRIND_MAKE_MEM_UNDEFINED(store_last_get[store_pool], size);
 /* Update next pointer and number of bytes left in the current block. */
 
 next_yield[store_pool] = (void *)((char *)next_yield[store_pool] + size);
 yield_length[store_pool] -= size;
 
-return store_last_get;
+return store_last_get[store_pool];
 }
 
 
@@ -265,6 +294,7 @@ DEBUG(D_memory)
 if (newsize % alignment != 0) newsize += alignment - (newsize % alignment);
 next_yield[store_pool] = (char *)ptr + newsize;
 yield_length[store_pool] -= newsize - rounded_oldsize;
+VALGRIND_MAKE_MEM_UNDEFINED(ptr + oldsize, inc);
 return TRUE;
 }
 
@@ -299,7 +329,7 @@ int newlength;
 
 /* Last store operation was not a get */
 
-store_last_get = NULL;
+store_last_get[store_pool] = NULL;
 
 /* See if the place is in the current block - as it often will be. Otherwise,
 search for the block in which it lies. */
@@ -323,11 +353,22 @@ newlength = bc + b->length - (char *)ptr;
 #ifndef COMPILE_UTILITY
 if (running_in_test_harness) memset(ptr, 0xF0, newlength);
 #endif
+VALGRIND_MAKE_MEM_NOACCESS(ptr, newlength);
 yield_length[store_pool] = newlength - (newlength % alignment);
 next_yield[store_pool] = (char *)ptr + (newlength % alignment);
 current_block[store_pool] = b;
 
-/* Free any subsequent blocks, if any */
+/* Free any subsequent block. Do NOT free the first successor, if our
+current block has less than 256 bytes left. This should prevent us from
+flapping memory. However, keep this block only when it has the default size. */
+
+if (yield_length[store_pool] < STOREPOOL_MIN_SIZE &&
+    b->next != NULL &&
+    b->next->length == STORE_BLOCK_SIZE)
+  {
+  b = b->next;
+  VALGRIND_MAKE_MEM_NOACCESS((char *)b + ALIGNED_SIZEOF_STOREBLOCK, b->length - ALIGNED_SIZEOF_STOREBLOCK);
+  }
 
 bb = b->next;
 b->next = NULL;
